import { Event } from "./Event.ts";

export class AsyncPool {
	limit;
	current = 0;
	current_changed = new Event<void>();

	constructor(limit: number = 8) {
		this.limit = limit;
	}

	async run<O>(task: () => Promise<O>) {
		while (this.current >= this.limit) await this.current_changed.wait();
		this.update_current(+1);
		const result = await task();
		this.update_current(-1);
		return result;
	}

	update_current(increment: number) {
		this.current += increment;
		this.current_changed.trigger();
	}
}
